function [BestFitness, BestProbeNumber, BestTimeStep, LastStep, R, M, Neval] = CFO(FunctionName, Np, Nd, Nt, XiMin, XiMax, InitialProbesPlaceMode, Gamma, Alpha, Beta, FrepInit, DeltaFrep, MinFrep)
% Main CFO function, a part of "CFOTool: a MATLAB toolbox for
% Central Force Optimization (CFO)" project
%
% Copyright (C) 2011  Hamed Abdy, Richard A. Formatto, Seyyed Hamid Zahiri
% 
% CFOTool is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CFOTool is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% MATLAB is a registered trademark of The MathWorks, Inc.
%
% For more information please see "README.txt" file that you recieved with
% this code.
% -------------------------------------------------------------------------
% Performs optimization using CFO algorithm
%
% Inputs:
% FunctionName: objective function
% Np: Number of probes
% Nd: Number of problem dimensions
% Nt: Number of iterations
% XiMin: Minimum value in dimensions matrix
% XiMax: Maximum value in dimensions matrix
% InitialProbesPlaceMode: Mode of placing initial probes:
%   'Uniform On-Axis': Places probes uniformly on “probe lines” parallel to
%       the coordinate axes, according to gamma.
%   '2D Grid': Places probes uniformly on a 2D grid.
%   'Uniform On-Diagonal': Places probes uniformly slightly off decision
%      space diagonal.
%   'Random': Places probes randomly.
% Gammas: Gamma value (see reference [6])
% Alpha: Alpha value (see reference [6])
% Beta: Beta value (see reference [6])
% FrepInit: Initial reposition factor
% DeltaFrep: Value used for changing the amount of reposition factor
% MinFrep: Minimum reposition factor
%
% Outputs:
% BestFitness: Best fitness seen among all times
% BestProbeNumber: Best probe number in which best fitness has seen among
% all times
% BestTimeStep: Best time step in which best fitness has seen among all
% times
% LastStep: Time step in which algorithm stops
% R: Probes matrix
% M: Fitnesses of probes matrix
% Neval: Number of evaluating objective function
%
% -------------------------------------------------------------------------

Neval = 0;

LastStep = Nt;

Nt = Nt + 1; % MATLAB doesn't support 0 as element#, so we add 1 to all times
ObjectiveFunction = str2func(FunctionName);


% STEP (A1) -------- Compute Initial Probe Distribution (Step 0)-----------
R = IPD(Np, Nd, Nt, XiMin, XiMax, Gamma, InitialProbesPlaceMode);

% STEP (A2) ---------- Compute Initial Fitness Matrix (Step 0) ------------
M = zeros(Np, Nt);
M(:, 1) = ObjectiveFunction(R(:, :, 1));
Neval = Neval + Np;

% STEP (A3) ------- Set Initial Probe Accelerations to ZERO (Step 0)-------
A = zeros(Np, Nd, Nt);

% STEP (A4) --------------------- Initialize Frep -------------------------
Frep = FrepInit;


% ======= LOOP ON TIME STEPS STARTING AT STEP #1 (#2 in this code) ========
BestFitness = M(1, 1);
BestProbeNumber = 1;
BestTimeStep = 0;

for j = 2:Nt
    
    % STEP (B) ---- Compute Probe Position Vectors for this Time Step -----
    R(:, :, j) = R(:, :, j - 1) + A(:, :, j - 1);
    
    % STEP (C) ---------------- Retrieve Errant Probes --------------------
    R = RetrieveErrantProbes(R, j, XiMin, XiMax, Frep);
    
    % STEP (D) --- Compute Fitness Matrix for Current Probe Distribution --
    M(:, j) = ObjectiveFunction(R(:, :, j));
    Neval = Neval + Np;
    
    % STEP (E) ------------------------------------------------------------
    % Compute Accelerations Based on Current Probe Distribution & Fitnesses
    % ---------------------------------------------------------------------
    for p = 1:Np
        for i = 1:Nd
            for k = 1:Np
                if k ~= p
                    SumSQ = sum((R(k, :, j) - R(p, :, j)) .^ 2);
                    if SumSQ ~= 0 % to avoid zero denominator
                        Denom = sqrt(SumSQ);
                        Numerator = UnitStep(M(k, j) - M(p, j)) * (M(k, j) - M(p, j));
                        A(p, i, j) = A(p, i, j) + (R(k, i, j) - R(p, i, j)) ...
                            * (Numerator ^ Alpha) / (Denom ^ Beta);
                    end
                end
            end
        end
    end
    
    % ------ Get Best Fitness & Corresponding Probe # and Time Step -------
    [tmpBestFitness, tmpBestProbeNumber] = max(M(:, j));
    if tmpBestFitness >= BestFitness
        BestFitness = tmpBestFitness;
        BestProbeNumber = tmpBestProbeNumber;
        BestTimeStep = j - 1;
    end
    
    % ------------------------ Increment Frep -----------------------------
    Frep = Frep + DeltaFrep;
    if Frep >= (1 + eps) % keep Frep in range [0.05,1]
        Frep = MinFrep;
    end
    
    % Starting at Step #20 Shrink Decision Space Around Best Probe Every
    % 20th Step
    if mod(j - 1, 20) <= eps && j >= 21
        % shrink DS by 0.5
        XiMin = XiMin + (R(BestProbeNumber, :, BestTimeStep + 1) - XiMin) ./ 2;
        XiMax = XiMax - (XiMax - R(BestProbeNumber, :, BestTimeStep + 1)) ./ 2;
        
        R = RetrieveErrantProbes(R, j, XiMin, XiMax, Frep);
        % TO RETRIEVE PROBES LYING OUTSIDE SHRUNKEN DS
    end
    
    % STEP (F) ------------- Check for Early Run Termination --------------
    if HasFITNESSsaturated(25, j, M)
        LastStep = j - 1;
        break;
    end
end
end
