function [R] = IPD(Np, Nd, Nt, XiMin, XiMax, Gamma, InitialProbesPlaceMode)
% Initial Probe Distribution function, a part of "CFOTool: a MATLAB toolbox
% for Central Force Optimization (CFO)" project
%
% Copyright (C) 2011  Hamed Abdy, Richard A. Formatto, Seyyed Hamid Zahiri
% 
% CFOTool is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CFOTool is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% MATLAB is a registered trademark of The MathWorks, Inc.
%
% For more information please see "README.txt" file that you recieved with
% this code.
% -------------------------------------------------------------------------
% Places initial probes on decision space using given method PlaceInitialProbes.
%
% Inputs:
% Np: Number of probes
%
% Nd: Number of dimensions
%
% InitialProbesPlaceMode: Mode of placing initial probes:
%   'Uniform On-Axis': Places probes uniformly on “probe lines” parallel to
%       the coordinate axes, according to gamma.
%   '2D Grid': Places probes uniformly on a 2D grid.
%   'Uniform On-Diagonal': Places probes uniformly slightly off decision
%      space diagonal.
%   'Random': Places probes randomly.
%
% Output:
% R: Probes matrix
%
% -------------------------------------------------------------------------

R = zeros(Np, Nd, Nt);

switch InitialProbesPlaceMode
    case 'Uniform On-Axis'
        if Nd > 1
            NumProbesPerDimension = fix(Np / Nd); % even #
        else
            NumProbesPerDimension = Np;
        end
        
        R(:, :, 1) = repmat(XiMin + Gamma .* (XiMax - XiMin), Np, 1);
        
        for i = 1: Nd % place probes probe line-by-probe line (i is dimension number)
            DeltaXi = (XiMax(i) - XiMin(i)) / (NumProbesPerDimension - 1);
            for k = 1:NumProbesPerDimension
                p = k + NumProbesPerDimension * (i - 1); % probe #
                R(p, i, 1) = XiMin(i) + (k - 1) * DeltaXi;
            end
        end
        
    case '2D Grid'
        NumProbesPerDimension = sqrt(Np);
        NumX1points = NumProbesPerDimension;
        NumX2points = NumX1points;
        DelX1 = (XiMax(1) - XiMin(1)) / (NumX1points - 1);
        DelX2 = (XiMax(2) - XiMin(2)) / (NumX2points - 1);
        
        for x1pointNum = 1:NumX1points
            for x2pointNum = 1:NumX2points
                p = NumX1points * (x1pointNum - 1) + x2pointNum;  % probe #
                R(p, 1, 1) = XiMin(1) + DelX1 * (x1pointNum - 1); % x1 coord
                R(p, 2, 1) = XiMin(2) + DelX2 * (x2pointNum - 1); % x2 coord
            end
        end
        
    case 'Uniform On-Diagonal'
        DeltaX = (XiMax - XiMin) ./ (Np - 1);
        R(:, :, 1) = repmat(XiMin, Np, 1) + ((1:Np) - 1)' * DeltaX;
        
    case 'Random'
        R(:, :, 1) = repmat(XiMin, Np, 1) + repmat((XiMax - XiMin), Np, 1) .* rand(Np, Nd);
    otherwise
        error('Invalid InitialProbesPlaceMode.');
end

end

