function [BestFitnessOverall, BestProbeNumberOveral, BestTimeStepOveral, BestNpNd, BestGamma, LastStepBestRun, Rbest, Mbest, Neval] = ParameterFreeCFO(FunctionName, Nd, Nt, XiMin, XiMax, NumGammas, Alpha, Beta, FrepInit, DeltaFrep, MinFrep)
% Parameter Free CFO function (see reference [10]), a part of "CFOTool: a
% MATLAB toolbox for Central Force Optimization (CFO)" project
%
% Copyright (C) 2011  Hamed Abdy, Richard A. Formatto, Seyyed Hamid Zahiri
% 
% CFOTool is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% CFOTool is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% You should have received a copy of the GNU General Public License
% along with this program.  If not, see <http://www.gnu.org/licenses/>.
%
% MATLAB is a registered trademark of The MathWorks, Inc.
%
% For more information please see "README.txt" file that you recieved with
% this code.
% -------------------------------------------------------------------------
% Performs optimization using CFO algorithm without need of some parameters
%
% Inputs:
% FunctionName: objective function
% Nd: Number of problem dimensions
% Nt: Number of iterations
% XiMin: Minimum value in dimensions matrix
% XiMax: Maximum value in dimensions matrix
% NumGammas: Number of gammas to be evaluated
% Alpha: Alpha value (see reference [6])
% Beta: Beta value (see reference [6])
% FrepInit: Initial reposition factor
% DeltaFrep: Value used for changing the amount of reposition factor
% MinFrep: Minimum reposition factor
%
% Outputs:
% BestFitnessOverall: Best fitness seen among all runs
% BestProbeNumberOveral: Best probe number in which best fitness has seen among all runs
% BestTimeStepOveral: Best time step in which best fitness has seen among all runs
% BestNpNd: Best Np per Nd in which best fitness has seen among all runs
% BestGamma: Best Gamma in which best fitness has seen among all runs
% LastStepBestRun: Last step of run in which best fitness has seen among all runs
% Rbest: Probes resulting best fitness
% Mbest: Fitnesses of Rbest
% Neval: Number of evaluating objective function
%
% -------------------------------------------------------------------------

Neval = 0;

if (Nd >=1 && Nd <= 6) % set Np/Nd based on Nd to avoid excessive runtimes
    MinProbesPerDimension = 2;
    MaxProbesPerDimension = 14;
elseif (Nd >= 7 && Nd <= 10)
    MinProbesPerDimension = 2;
    MaxProbesPerDimension = 12;
elseif (Nd >= 11 && Nd <= 15)
    MinProbesPerDimension = 2;
    MaxProbesPerDimension = 10;
elseif (Nd >= 16 && Nd <= 20)
    MinProbesPerDimension = 2;
    MaxProbesPerDimension = 8;
elseif (Nd >= 21 && Nd <= 30)
    MinProbesPerDimension = 2;
    MaxProbesPerDimension = 6;
else
    MinProbesPerDimension = 2;
    MaxProbesPerDimension = 4;
end

BestFitnessOverall = -Inf;

for NumProbesPerDimension = MinProbesPerDimension:2:MaxProbesPerDimension
    Np = NumProbesPerDimension * Nd;
    
    for GammaNumber = 1:NumGammas
        Gamma = (GammaNumber - 1) / (NumGammas - 1);

        [BestFitnessThisRun, BestProbeNumberThisRun, BestTimeStepThisRun, LastStep, R, M, tmpNeval] = CFO(FunctionName, Np, Nd, Nt, XiMin, XiMax, 'Uniform On-Axis', Gamma, Alpha, Beta, FrepInit, DeltaFrep, MinFrep);
        
        Neval = Neval + tmpNeval;
        
        % ------- Best Overall Fitness & Corresponding Parameters ---------
        if BestFitnessThisRun >= BestFitnessOverall
            BestFitnessOverall = BestFitnessThisRun;
            BestProbeNumberOveral = BestProbeNumberThisRun;
            BestTimeStepOveral = BestTimeStepThisRun;
            BestNpNd = NumProbesPerDimension;
            BestGamma = Gamma;
            LastStepBestRun = LastStep;
            Rbest = R;
            Mbest = M;
        end
        
        % STEP (G) -- Reset Decision Space Boundaries to Initial Values ---
        % There is no need to this step, because we used CFO as a function,
        % so Decision Space Boundaries will return to their initial value
        % for next run.
    end
end

end
